#  Copyright 2016-2020 Prasanna Pendse <prasanna.pendse@gmail.com>
# 
#  This file is part of power-daps.
# 
#  power-daps is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
# 
#  power-daps is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with power-daps.  If not, see <https://www.gnu.org/licenses/>.

from dap_core.base_actions.deps_action_base import DepsActionBase


class DepsAction(DepsActionBase):
  def __init__(self, dependencies_file_location=""):
    super().__init__(dependencies_file_location)

  def run(self):
    super().run()
    return 0, ""


def action():
  return DepsAction()



#  Copyright 2016-2020 Prasanna Pendse <prasanna.pendse@gmail.com>
#
#  This file is part of power-daps.
#
#  power-daps is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  power-daps is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with power-daps.  If not, see <https://www.gnu.org/licenses/>.


from dap_core import common


def classpath_string(classpath=""):
  if classpath != "":
    cp_string = " -cp " + classpath + ":" + libs_classpath()
  else:
    cp_string = " -cp " + libs_classpath()
  return cp_string


def libs_classpath():
  libs = common.run_command_in_shell('find lib/java -type f -name "*.jar" -print')[1]
  return ":".join(libs.splitlines())


def production_classpath():
  return "target/production"


def test_classpath():
  return "target/test"


def list_of_test_classes(test_source_dir):
  test_classes = common.run_command_in_shell('find ' + test_source_dir +  ' -type f -name "*Test.java" -print')[1].splitlines()
  return [test_class.replace(test_source_dir + "/", "").replace(".java", "").replace("/", ".") for test_class in test_classes]




#  Copyright 2016-2020 Prasanna Pendse <prasanna.pendse@gmail.com>
#
#  This file is part of power-daps.
#
#  power-daps is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  power-daps is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with power-daps.  If not, see <https://www.gnu.org/licenses/>.
#
#  This file is part of power-daps.
#
#  power-daps is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  power-daps is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with power-daps.  If not, see <https://www.gnu.org/licenses/>.

import os
from dap_core import common
from power_daps.java import java_helper

class JavaCommand:
  def __init__(self, main_class, args_to_main_class=[], classpath=""):
    self.main_class = main_class
    self.args_to_main_class = args_to_main_class
    self.classpath = classpath
    self.java_opts = os.getenv("JAVA_OPTS", "")
    common.print_raw("JAVA_OPTS=" + self.java_opts)


  def run(self):
    cp_string = java_helper.classpath_string(self.classpath)
    command_array = []
    command_array.append(common.which('java'))
    command_array.append(cp_string.lstrip())
    if self.java_opts:
      command_array.append(self.java_opts)
    command_array.append(self.main_class)
    command_array.extend(self.args_to_main_class)

    run_unit_test_command = " ".join(command_array).split(" ")

    exit_code, output = common.run_with_io(run_unit_test_command)
    try:
      output = output.decode()
    except (UnicodeDecodeError, AttributeError):
      pass
    # common.print_raw(output)
    return exit_code, output





#  Copyright 2016-2020 Prasanna Pendse <prasanna.pendse@gmail.com>
#
#  This file is part of power-daps.
#
#  power-daps is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  power-daps is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with power-daps.  If not, see <https://www.gnu.org/licenses/>.

import setuptools


def generate_glob_patterns(depth):
  glob_patterns = ["*", ".*"]
  for i in range(depth - 1):
    glob_patterns_to_add = []
    for p in glob_patterns:
      glob_patterns_to_add.append(p + "/*")
      glob_patterns_to_add.append(p + "/.*")
    glob_patterns += glob_patterns_to_add

  return glob_patterns


with open("README.md", "r") as fh:
  long_description = fh.read()


package_data_patterns = generate_glob_patterns(6)

setuptools.setup(
  name="power-daps-meta-model-java-9",
  version="0.0.6",
  author="Prasanna Pendse",
  author_email="prasanna.pendse@gmail.com",
  description="Java meta-model for power-daps - a build tool that builds apps in multiple languages",
  long_description=long_description,
  long_description_content_type="text/markdown",
  url="https://github.com/power-daps/power-daps",
  package_dir={"": "src"},
  packages=setuptools.find_packages(where="src"),
  entry_points = {'power_daps.meta_model.actions': [
    'power_daps.java.actions=power_daps.java.actions']},
  package_data = {
    "power_daps.java.templates": package_data_patterns, # ["**/*", "**/**/**", "**/.*", "*/.*/*", "*/**/.*"],
  },
  classifiers=[
    "Programming Language :: Python :: 3",
    "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
    "Operating System :: OS Independent",
  ],
  python_requires='>=3.6',
)



#  Copyright 2016-2020 Prasanna Pendse <prasanna.pendse@gmail.com>
#
#  This file is part of power-daps.
#
#  power-daps is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  power-daps is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with power-daps.  If not, see <https://www.gnu.org/licenses/>.

from dap_core import common
from dap_core.base_actions.dap_action_base import DapActionBase
from power_daps.java import java_helper
from power_daps.java.java_command import JavaCommand


class RunAction(DapActionBase):

  def __init__(self):
    super().__init__()

  def run(self):
    super().run()
    main_files = common.find_files_containing("public static void main(")
    common.print_verbose(main_files)

    for main_file in main_files:
      main_class = main_file.replace("./src/", '').replace('.java', '').replace('/', '.')
      JavaCommand(main_class, [], java_helper.production_classpath()).run()

    return 0, ""

def action():
  return RunAction()




#  Copyright 2016-2020 Prasanna Pendse <prasanna.pendse@gmail.com>
#
#  This file is part of power-daps.
#
#  power-daps is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  power-daps is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with power-daps.  If not, see <https://www.gnu.org/licenses/>.

from dap_core import common
from power_daps.java import java_helper

class JavacCommand:
  def __init__(self, source_dir, target_dir, classpath):
    self.source_dir = source_dir
    self.target_dir = target_dir
    self.classpath = classpath

  def run(self):
    cp_string = java_helper.classpath_string(self.classpath)

    common.run_command_in_shell('mkdir -p ' + self.target_dir)

    exit_code, output = common.run_command_in_shell(
      'find ' + self.source_dir + ' ' +
      '-type f -name "*.java" -print | xargs ' + common.which('javac') + ' ' +
      cp_string + " " +
      '-d ' + self.target_dir + ' -sourcepath ' + self.source_dir)
    try:
      output = output.decode()
    except (UnicodeDecodeError, AttributeError):
      pass

    return exit_code, output





#  Copyright 2016-2020 Prasanna Pendse <prasanna.pendse@gmail.com>
# 
#  This file is part of power-daps.
# 
#  power-daps is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
# 
#  power-daps is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with power-daps.  If not, see <https://www.gnu.org/licenses/>.

from dap_core.base_actions.dap_action_base import DapActionBase
from power_daps.java.javac_command import JavacCommand


class CompileTestAction(DapActionBase):

  def __init__(self, source_dir="test", target_dir="target/test", classpath=".:target/production"):
    super().__init__()
    self.source_dir = source_dir
    self.target_dir = target_dir
    self.classpath = classpath

  def run(self):
    super().run()
    JavacCommand(self.source_dir, self.target_dir, self.classpath).run()
    return 0, ""


def action():
  return CompileTestAction()
